# Copies epoch files within specified range.
# Epoch is 1900 (in milliseconds)
# If no supplied end date, copy one epoch file nearest to start date.
# Arg 1: Source directory
# Arg 2: Target directory
# Arg 3: Prefix of the epoch files. i.e. <prefix><epoch>
# Arg 4: Start date. In <yyyy>-<mm>-<dd> format.
# Arg 5: (Optional) End date. In <yyyy>-<mm>-<dd> format.
# Arg 6: (Optional) Filename. New filename of the copied epoch file.
#         Only works when no end date.
# 
# Author: Ivan Dustin B. Bilon
######################
use Time::Local;
use File::Copy;
use File::Glob ':glob';
######################
my $from = shift;
my $to = shift;
my $prefix = shift;
my $date = shift;
my $end_date = shift;
my $filename = shift;
######################
my $e1900 = 2209075200; # offset in seconds, 1900 - 1970 epoch
my @date = split(/-/, $date);
my $epoch = (timelocal(0,0,0,@date[2],@date[1]-1,@date[0]));
my $end_epoch;
if ( $end_date ne "" ) {
    my @end_date = split(/-/, $end_date);
    $end_epoch = (timelocal(59,59,23,@end_date[2],@end_date[1]-1,@end_date[0]));
}
######################
# FIND FILES
my @files = bsd_glob "$from/$prefix*";
if( scalar(@files) == 0 ) {
    exit 0;
}
my @epochs;
foreach my $file (@files) {
    my @a = split(/\./, $file);
    my $e = pop(@a);
    # my $e = substr($file, index($file, $prefix) + length($prefix));
    push(@epochs, $e);
}
######################
# SORT EPOCHS, ASCENDING
my @sorted = sort {$a <=> $b} @epochs;
######################
# FIND THE FILE
if ( $end_epoch ) {
    foreach my $e (@sorted) {
        if ( ($e >= $epoch) && ($e <= $end_epoch) ) {
            my $file = bsd_glob "$from/$prefix$e";
            if ( -e $file ) {
                copy($file, $to) or die "cannot copy file $file. $!";
            }
        }
    }
} else {
    my $found;
    foreach my $e (@sorted) {
        $found = $e;
        if ( $e > $epoch ) {
            last;
        }
    }
    my $file = bsd_glob "$from/$prefix$found";
    if ( -e $file ) {
        copy($file, "$to/$filename") or die "cannot copy file $file. $!";
    }
}
