#!/etc/opt/openit/perl -w

use strict;
use Time::Local;
use IO::Zlib;
use Getopt::Long;

my $curr;
my $prev;

my $MIDNIGHT;
my $START_TIME = "1970-01-01 00:00:00";
my $count = 0;
my $newcount = 0;
my $login_count = 0;
my $logout_count = 0;
my $timeout_count = 0;

my %Concurrency = ();
my %ProductCount = (); 	# Full product version names
my %ProdCount = ();		# Just the main product name, without version info

my %ActiveLicenses = ();# List of starting times for licenses for a particular 
						# product/version/user/machine/account combination

my %ProdErrorReported = ();

my $opt_verbose;
unless ( GetOptions("verbose" => \ $opt_verbose)) {}

sub dump_status
{
	my ($midnight) = @_;
	#printf(STDERR "Status at %s (%d/%d/%d login/logout/timeout)\n", $midnight, $login_count, $logout_count, $timeout_count);
	my @keys = sort keys %ActiveLicenses;
	my $dd = iso2dmy($midnight);
	
	my $active = 0;
	foreach (@keys) { $active += scalar(@{$ActiveLicenses{$_}});}
	#printf(STDERR "%d active licenses\n", $active);
	foreach (@keys) {
		my $cnt = scalar(@{$ActiveLicenses{$_}});
		for (my $c = 1; $c <= $cnt; $c++) {
			#printf("Logout\t%s\t00:00:00\t%s\t\"00:00:00\"\n", $dd, $_);
		}
	}
	foreach (reverse @keys) {
		my $cnt = scalar(@{$ActiveLicenses{$_}});
		for (my $c = 1; $c <= $cnt; $c++) {
			#printf("Login\t%s\t00:00:00\t%s\t\%d\n", $dd, $_, $c);
		}
	}
}

sub prodname
{
	my ($product) = @_;
#	if (/^([^-]+)\d*\-(.+)$/) {
#		$prod = $1;
#	}
	if (!defined($product) || $product eq '') {
		#warn("Bad prodname input!");
		return "";
	}
	my ($prod,$ver);
	($prod, $ver) = split(/-/, $product);
	if ($prod =~ /(.+)64$/) {
		$prod = $1;
	}
	return $prod;
}

my %lines = (); # Indexed by log file timestamp!

my $curr_time = "1970-01-01 00:00:00.000";
my $START_DATE;
my $LAST_VALID_TIME = "9999-99-99 99:99:99";

# Fix missing Login messages:
my %missing_login = ();
my %login = ();
my %checkout = ();

my $indir = shift;
$indir = '.' unless (defined($indir));

my $outdir = shift;
$outdir = $indir unless (defined($outdir));

my $license_usage_dir = shift;
$license_usage_dir = $indir unless (defined($license_usage_dir));

my $time_cutoff = shift;
$START_DATE = $time_cutoff if (defined($time_cutoff));

$time_cutoff = shift;
if (defined($time_cutoff)) {
	$LAST_VALID_TIME = $time_cutoff;
	$LAST_VALID_TIME .= "23:59:59" if (length($LAST_VALID_TIME) < 19);
}

sub dmy2iso
{
	my ($dmy) = @_;
	return sprintf("%d-%s-%s", substr($dmy,6,2)+2000, substr($dmy, 3,2), substr($dmy, 0,2));
}

sub iso2dmy
{
	my ($iso) = @_;
	return sprintf("%s/%s/%s", substr($iso,8,2), substr($iso, 5,2), substr($iso, 2,2));
}

sub iso2sec
{
	($_) = @_;
	if (/(\d\d\d\d)\-(\d\d)\-(\d\d)\s+(\d\d)\:(\d\d)\:(\d\d)/) {
		return timegm($6, $5, $4, $3, $2-1, $1);
	}
	warn("Bad date format: $_", $_);
}

sub sec2iso
{
	my ($t) = @_;
	my ($sec, $min, $hour, $day, $mon, $year) = gmtime($t);
	return sprintf("%d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub iso2epoch
{
	my ($i) = @_;
	return 1000*(iso2sec($i) + (70*365+16)*86400); # 1900-01-01 to 1970-01-01 contains 16 leap years!
}

sub epoch2iso
{
	my ($i) = @_;
	return sec2iso(int($i/1000) - (70*365+16)*86400);
}

sub hms2sec
{
	($_) = @_;
	if (defined($_) && /^(\d+)\:(\d\d)\:(\d\d)$/) {
		if ($1 > 23) {
			#printf(STDERR "Long license time: %s\n", $_);
		}
		return $1*3600+$2*60+$3;
	}
	#warn("Bad hh:mm:ss format: $_", $_);
	return 0;
}

sub sec2hms
{
	my ($s) = @_;
	my $m = int($s / 60);
	$s -= $m*60;
	my $h = int($m / 60);
	$m -= $h*60;
	return sprintf("%02d:%02d:%02d", $h, $m, $s);
}

sub delta
{
	my ($prev, $curr) = @_;
	return iso2sec($curr) - iso2sec($prev);
}

sub gatherfilenames
{
	my ($indir, $pattern) = @_;
	die("Missing pattern") unless defined($indir);
	opendir(D, $indir);
	my @f = sort readdir(D);
	closedir(D);
		
	my @files = ();
	my @epoch = ();
	my $onefile = 0;
	if (scalar(@f) == 0 && -f $indir) {
		push(@f, $indir);
		$onefile = 1;
	}
	foreach (@f) {
		next if (/^\.+$/);

		my $path = $indir;
		$path .="/".$_ unless $onefile;
		if (-d $path) {
			my @sub = gatherfilenames($path, $pattern);
			push(@files, @sub);
		}
		elsif (qr/$pattern/) {
			push(@files, $path);
		}
	}
	@files = sort(@files);
	return @files;
}

my @files = gatherfilenames($indir, '\.in[\.gz]$');
@files = &fix_file_timestamps(@files);
my @snapshot = gatherfilenames($license_usage_dir, 'license-usage-nethasp\.\d+[\.gz]$');
@snapshot = &fix_snapshot_timestamps(\@files, @snapshot);

sub gzopen
{
	my ($fname) = @_;
	my $fh;
	if ($fname =~/\.gz$/) { $fh = IO::Zlib->new($fname,"r"); } else { open($fh, "<", $fname); }
	return $fh;
}

#Timestamp fixups, first for the .in files:
sub fix_file_timestamps
{
	my @ff = @_;
	my @f = ();
	foreach (@ff) {
		if (!(/\d{13}/)) {
			push(@f, $_);
			next;
		}
		
		my $fname = $_;
		my ($pre, $post) = ($`, $');
		my ($fh, $ldate, $fdate);
		$fh = gzopen($fname);
		while (<$fh>) {
			chomp;
			if (/(\d\d\/\d\d\/\d\d)\s+(\d\d:\d\d:\d\d)/) {
				$ldate = dmy2iso($1)." ".$2;
				$fdate = $ldate unless (defined($fdate));
			}
		}
		close($fh);

		if (defined($ldate)) {
			my $nname = sprintf("%s%d-%d%s", $pre, iso2sec($fdate), iso2sec($ldate), $post);
			printf(STDERR "Renaming %s to %s\n", $fname, $nname) if(defined($opt_verbose));
			rename($fname, $nname);
			push(@f, $nname);
		}
	}
	return sort @f;
}

#Next we take any usage/snapshot files
sub fix_snapshot_timestamps
{
	my ($files, @ss) = @_;
	my @f = ();
	foreach (@ss) {
		if (!(/(\d{13})/)) {
			push(@f, $_);
			next;
		}

		my $fname = $_;
		my ($pre, $post) = ($`, $');
		my ($fh, $ldate, $fdate);
		$fh = gzopen($fname);
		while (<$fh>) {
			chomp;
			if (/license_start.(\d{13})/) {
				$ldate = epoch2iso($1);
				$fdate = $ldate unless (defined($fdate));
			}
		}
		close($fh);
		if (defined($ldate)) {
			my $unix = iso2sec($ldate);
			# Try to locate the first .in file which ends >= the final usage record and use the ending time of that file:
			foreach (reverse sort @{$files}) {
				if (/(\d{10})-(\d{10})/) {
					if ($2 >= $unix) {
						$unix = $2;
					}
				}
			}
			my $nname = sprintf("%s%d%s", $pre, $unix, $post);
			printf(STDERR "Renaming %s to %s\n", $fname, $nname) if(defined($opt_verbose));
			rename($fname, $nname);
			push(@f, $nname);
		}
	}
	return sort @f;
}

#parse_license_status_file($indir);

printf(STDERR "Selecting files to process:\n") if(defined($opt_verbose));
# Find the last snapshot file (if any) which is <= $START_DATE (which can be null in which case we process all files)
my $snapshot = ''; # snapshot file to use
my $in = 0; # Index of .in file to start with

my $first_file = scalar(@snapshot); # In sorted order...
while ($first_file--) {				# Starting from the last!
	my $file = $snapshot[$first_file];
	my $dato;
	if ($file =~ /license-usage-\S+\.(\d{13})$/) {
		$dato = epoch2iso($1);
	}
	elsif ($file =~ /\D(\d{10})$/) {
		$dato = sec2iso($1);
	}
	if (defined($dato)) {
		if (!defined($START_DATE) || $dato le $START_DATE) {
			$snapshot = $file;
			$START_DATE = $dato;
			printf(STDERR "Found snapshot from %s: %s\n", $dato, $snapshot) if(defined($opt_verbose));
			last;
		} 
	}
}

my $dosnapshot = $snapshot ne '';
parse_license_status_file($snapshot) if ($dosnapshot);

# Filter file list, .in[.gz]? after the snapshot
my @tempfiles = ();
foreach (@files) {
	if (/(\d{10})\D(\d{10})\D/) {
		my ($bdato, $edato) = ($1, $2);
		my $dato = sec2iso($bdato);
		push(@tempfiles, $_) if (!$dosnapshot || $dato ge $START_DATE);
	}
	else {
		push(@tempfiles, $_);
	}
}
@files = @tempfiles;

my %patches = ();
my %active_product = (); # Stores all currently active licenses
my %use = ();

my %matched_lines = ();

my $missing_login = 0;
my $missing_logout = 0;
foreach (@files) {
	printf(STDERR "Reading %s\n", $_) if(defined($opt_verbose));
	my $fh = gzopen($_);
	my $hdr = 1;
	while (<$fh>) {
		chomp;
		if ($hdr) {
			if ($_ eq '@META:') {
				while (<$fh>) {
					chomp;
					last if ($_ eq '@DATA:');
				}
				$_ = <$fh>; 
				chomp;
			}
			$hdr = 0;
		}
		next if (/^\s*$/); # Blank lines should be skipped!
		last if (/^\@SIGNATURE:/);

	# Save (time-)sorted list of all found lines!
		if (/(\d\d\/\d\d\/\d\d)\s+(\d\d:\d\d:\d\d)/) {
			$curr_time = dmy2iso($1)." ".$2;
		}
		next unless ($dosnapshot || $curr_time ge $START_TIME);
		# Skip the rest if past the last date we are interested in...
		last if ($curr_time gt $LAST_VALID_TIME);

		my $line = $_;
		my @f = split(/\t/, $_);

		my $prod = prodname($f[3]);

		if ($f[0] =~ /^(Login|Checkout)/i) {
	# Login	02/12/15	16:01:55	GAP-v9.0	co24672	NoAccount	ENVS0Z0C	04721		1
	# Checkout	05/02/16	12:35:12	PROSPER-v14.0	en29761	NoAccount	WPWS01368623	04721	10644	15-02-2016
			#my $curr = sprintf("%s %s", dmy2iso($f[1]), $f[2]);
			$use{$prod}++;
			my $key = join("\t", (@f)[3..7]);
			if  ($f[0] =~ /Checkout/i) {
				push(@{$checkout{$prod}{$key}}, join("\t", (@f)[8,9]));
			}
			else {
				if ($f[9] < $use{$prod}) { # Lost Logout message(s) before this?
					$missing_logout += $use{$prod}-$f[9];
					printf(STDERR "%d Logout(s) missing:\n%s\n", $use{$prod}-$f[9], $line) if(defined($opt_verbose));
					#substr($curr_time,10,1) = 'E';
				}
				push(@{$login{$prod}{$key}}, $curr_time);
				$use{$prod} = $f[9];
			}
		}
		elsif ($f[0] =~ /^(Logout|Checkin|Timeout)/i) {
	# Logout	20/01/16	15:40:20	OPENSERVER-v1.0	en33555	NoAccount	ENWS08498037	04721	00:00:44
	# Timeout	11/12/15	11:18:31	PROSPER-v12.0	dzb14026	NoAccount	DZHWS1090	04322	00:47:01
	# Checkin	25/01/16	07:41:08	PROSPER-v14.0	en29761	NoAccount	WPWS01368623	04721	10644	08-02-2016
			if (scalar(@f) < 9) {
				if (scalar(@f) == 8) {
					push(@f, "00:00:00");
				}
				else {
					printf(STDERR "??? $_\n") if(defined($opt_verbose));
					next;
				}
			}
			my $key = join("\t", (@f)[3..7]);
			my $found = 0;
			my ($payload, $start_dmy, $start_hms, $match);
			my @searchspace = (0,-1,1,-2,2);
			foreach (@searchspace) {
				my $offs = $_;
				my @temp = ();
				if ($f[0] =~ /^Checkin/i) {
					$payload = join("\t",(@f)[8,9]);
					foreach (@{$checkout{$prod}{$key}}) {
						if ($found == 0 && $_ eq $payload) {
							$found = 1;
							$matched_lines{$line}++;
						}
						else {
							push(@temp, $_);
						}
					}
					@{$checkout{$prod}{$key}} = @temp;
					last if ($found);
				}
				else { # Logout
					my $curr = sprintf("%s %s", dmy2iso($f[1]), $f[2]);
					my $currsec = iso2sec($curr);
					my $delta = hms2sec($f[8]);
					my $start_iso =  sec2iso($currsec - $delta + $offs);
					$start_dmy = iso2dmy($start_iso);
					$start_hms = substr($start_iso,-8,8);
					$match = $start_dmy."\t".$start_hms unless (defined($match));
					$payload = $start_iso;
					foreach (@{$login{$prod}{$key}}) {
						if ($found == 0 && $_ eq $payload) {
							$found = 1;
							if ($offs != 0) {
								$f[8] = sec2hms($delta-$offs);
								$line = join("\t", @f);
							}
							$matched_lines{$line}++;
							if (scalar(@{$login{$prod}{$key}}) > 1) {
								#printf(STDERR "%s\nMatched a login with multiple entries: %s \n", $line, join("\t",@{$login{$prod}{$key}}));
							}
						}
						else {
							push(@temp, $_);
						}
					}
					@{$login{$prod}{$key}} = @temp;
					last if ($found);
				}
			}
			if (!$found) {
				#printf(STDERR "No matching record found:\n%s\n", $line);
				if ($f[0] =~ /^Checkin/i) {
					warn("Checkin with no matching Checkout, cannot be fixed!\n$line\n");
				}
				else {
					my $mline = join("\t", "Login", $match, $key, "", "", $curr_time);
					printf(STDERR "Missing Login record inserted:\n%s\n%s\n", $mline, $line) if(defined($opt_verbose));
					my ($dmy, $hms) = split(/\t/, $match);
					my $iso = dmy2iso($dmy)." ".$hms;
					push(@{$missing_login{$iso}}, $mline);
					push(@{$lines{$iso}}, $mline);
					push(@{$patches{$iso}}, $mline);
					$missing_login++;
					# push(@{$patches{$iso}}, $line);
				}
			}
			$use{$prod}--;
		}
		
#Save all input as a hash of arrays, one array for each second with at least one line
		push(@{$lines{$curr_time}}, $line);
		
	}
	close($fh);
}
printf(STDERR "%d .in files loaded, %d Logout without Login, %d missing Logout\n", scalar(@files), $missing_login, $missing_logout) if(defined($opt_verbose));

# Generate a list of all logins that are still active, they should be matched with 
# patched-in Logout messages. We do that by cleaning up the %login hash:

my %lost_logins = (); # A hash (prog) of arrays
{
#	my %tlogin = ();
	my $active = 0;
	foreach (keys %login) {
		my $p = $_;
		foreach (keys %{$login{$p}}) {
			my $k = $_;
			my $a = scalar(@{$login{$p}{$k}});
#			@{$tlogin{$p}{$k}} = @{$login{$p}{$k}} if ($a);
			$active += $a;
			foreach (@{$login{$p}{$k}}) {
				my $iso = $_;
				my $dmy = iso2dmy($iso);
				my $hms = substr($iso,-8);
				push(@{$lost_logins{$p}}, join("\t", $iso, $k));
			}
		}
		my $lost = defined ($lost_logins{$p}) ? scalar( @{ $lost_logins{$p} } ) : 0;
		printf(STDERR "%s: %d lost sessions\n", $p, $lost) if(defined($opt_verbose));
	}
#	%login = %tlogin;
	printf(STDERR "There are %d active sessions\n", $active) if(defined($opt_verbose));
}
printf(STDERR "%d missing Logout detected\n", $missing_logout) if(defined($opt_verbose));
#exit(0);

my %lastuse = (); # Remembers the last time we saw each product
my $lastactivity = ''; # Global last timestamp, reset at each Startup
%use = ();
my %prev_use = ();
my @startup = ();
my %prev_log = ();
my %check = ();

sub check_status
{
#	foreach (keys %log) {
#		my $p = $_;
#		my $pc = 0;
#		foreach (keys %{$log{$p}}) {
#			my $k = $_;
#			$pc += scalar(@{$log{$p}{$k}});
#		}
#		$use{$p} = 0 unless (defined($use{$p}));
#		printf("%s: Found %d login records, expected %d\n", $p, $pc, $use{$p});
#	}
}

my $last_startup = '';

my @keys_lines = sort keys %lines;			# Also keep the input data in time order, should be OK already!
foreach (@keys_lines) {
	my $file_iso = $_;
	foreach (@{$lines{$_}}) {
		my $line = $_;
		if (/^Startup/) {
			foreach (keys %lastuse) {
				$lastuse{$_} = $lastactivity;
			}
			foreach (keys %use) {
				my $p = $_;
				$prev_use{$p} += $use{$p};
				$use{$p} = 0;
#				@{$prev_log{$p}} = @{$login{$p}};
			}
			$last_startup = $file_iso;
			#printf(STDERR "Startup seen $file_iso, reset %d lastuse counters to %s\n", scalar(keys %lastuse), $lastactivity);
			check_status();
			next;
		}
		# Calculate the correct running login count for all products

		my @f = split(/\t/, $_);
		next unless (scalar(@f) > 7);
		next unless ($f[1] =~ /\d\d\/\d\d\/\d\d/);

		my $prod = prodname($f[3]);
		my $key = join("\t", (@f)[3..7]);

		if ($f[0] =~ /^Login/i) {
			my $curr = $f[9];
			$use{$prod} = 0 unless (defined($use{$prod}));
			my $expt = $use{$prod}+1;
			$curr = $expt unless ($curr); # Inserted records have no count, so assume it to be correct!

			while ($curr < $expt) { # Must be a missing Logout before this one!
				# Pick the oldest session from %lost_logins
				my @temp = defined($lost_logins{$prod}) ? sort @{$lost_logins{$prod}} : ();
				$lastuse{$prod} = $file_iso unless (defined($lastuse{$prod}));
				my $missing_iso = $lastuse{$prod};
				my $slutt_dmy = iso2dmy($missing_iso);
				my $slutt_hms = substr($missing_iso,-8,8);
				my $line;
				my $oldest;
				if (scalar(@temp) && (split("\t", $temp[0]))[0] le $file_iso) {
					$oldest = shift @temp;
					my @k;
					($oldest, @k) = split("\t", $oldest);
					@{$lost_logins{$prod}} = @temp;
					$line = join("\t","Logout", $slutt_dmy, $slutt_hms, @k, sec2hms(iso2sec($missing_iso)-iso2sec($oldest)));
					$expt--;
				}
				else {
					$line = join("\t","Logout", $slutt_dmy, $slutt_hms, $prod, "Unknown", "Unknown", "Unknown", "Unknown", '00:00:00', 
						$expt, $curr);
					$expt--; # = $curr;
				}
				push(@{$lines{$missing_iso}}, $line);
				push(@{$patches{$missing_iso}}, $line);
				printf(STDERR "%s\n", $line) if(defined($opt_verbose));
			}
			$prev_use{$prod} += 0;
			while ($curr > $expt + $prev_use{$prod}) { # Must be a missing Login _just_ before this one, with no corresponding Logout in the file
				my $start_dmy = iso2dmy($file_iso);
				my $start_hms = substr($file_iso,-8,8);
				my $line = join("\t","Login", $start_dmy, $start_hms, $prod, "Unknown", "Unknown", "Unknown", "Unknown", '',$expt);
				push(@{$lines{$file_iso}}, $line);
				push(@{$patches{$file_iso}}, $line);
				my $k = join("\t",$prod, "Unknown", "Unknown", "Unknown", "Unknown");
				#push(@{$log{$prod}{$k}}, $file_iso);
				push(@{$lost_logins{$prod}}, $file_iso."\t".$k);
				printf(STDERR "%s\n", $line) if(defined($opt_verbose));
				$expt++; # = $curr;
			}
			if ($curr > $expt && ($prev_use{$prod} >= ($curr-$expt))) {
				$prev_use{$prod} -= ($curr-$expt);
			}
			$use{$prod} = $curr;
			$lastuse{$prod} = $file_iso;
			$lastactivity = $file_iso;
			#push(@{$log{$prod}{$key}}, $file_iso);
		}
		elsif ($f[0] =~ /^Checkout/i) {
			$use{$prod}++;
			#$lastuse{$prod} = $file_iso;
			$lastactivity = $file_iso;
		}
		elsif ($f[0] =~ /^(Logout|Timeout|Checkin)/i) {
			my $curr = $file_iso;
			my $currsec = iso2sec($curr);
			my $delta = hms2sec($f[8]);
			my $start_iso =  sec2iso($currsec - $delta);

			if ($start_iso lt $last_startup && $prev_use{$prod}) { 
				$prev_use{$prod}--;
			}
			else {
				$use{$prod}--;
				while ($use{$prod} < 0) {
					warn($f[0]."\nNegative use for $prod ".$use{$prod});
					my $start_dmy = iso2dmy($file_iso);
					my $start_hms = substr($file_iso,-8,8);
					my $line = join("\t","Login", $start_dmy, $start_hms, $prod, "Unknown", "Unknown", "Unknown", "Unknown", '',);
					push(@{$lines{$file_iso}}, $line);
					push(@{$patches{$file_iso}}, $line);
					printf(STDERR "%s\n", $line) if(defined($opt_verbose));
					$use{$prod}++;
				}
			}
			#$lastuse{$prod} = $file_iso;
			$lastactivity = $file_iso;
		}
	}
}

foreach (keys %use) {
	my $p = $_;
	my $active = $use{$p}+$prev_use{$p};
	printf(STDERR "%s: %d active (%d from before last Startup)\n", $p, $active, $prev_use{$p}) if(defined($opt_verbose));
}

sub create_path
{
	my ($path) = @_;
	my @parts = split(/[\/\\]/, $path);
	for (my $p = 0; $p < scalar(@parts)-1; $p++) {
		my $dir = join("/", @parts[0..$p]);
		mkdir($dir) unless (-d $dir);
	}
}

sub update_if_modified
{
	my ($path, @lines) = @_;
	create_path($path);
	my @f = ();
	if (-f $path) {  # File already exists, check if it has been updated:
		if (open(F , "<", $path)) {
			while (<F>) {
				chomp;
				push(@f, $_);
			}
			close(F);
		}
		if (join("\n", @f) eq join("\n", @lines)) {
			return 0; # No file update!
		}
	}
	if (-f $path || scalar(@lines)) {
		open(F, ">", $path) || die("Could not open $path for writing!");
		foreach (@lines) {
			printf(F "%s\n", $_);
		}
		close(F);
	}
}

my %patchlist;
if (scalar(keys %patches) == 0) {
	printf(STDOUT "No missing records found, no patch needed\n");
	exit 0;
}

my ($bdato, $edato) = (sort keys %patches)[0,-1];
my $patchfile = sprintf("%s/patch-%d-%d", $outdir, iso2sec($bdato), iso2sec($edato));

my @out = ();
foreach (sort keys %patches) {
	my $curr_time = $_;
	foreach (@{$patches{$curr_time}}) {
		next unless (defined($_) && $_);

		push(@out, $_);
	}
}
update_if_modified($patchfile, @out);

# Get rid of any extra patch files:
opendir(D, $outdir); my @patch_files = readdir(D); closedir(D);

foreach (@patch_files) {
	if (/^patch-\d+-\d+$/) {
		if ($outdir.'/'.$_ gt $patchfile) {
			unlink("$outdir/$_");
			printf(STDERR qq(unlink("$outdir/$_");\n)) if(defined($opt_verbose));
		}
	}
}

printf("reprocess: %s %s\n", $bdato, $edato);
exit(1);

sub parse_license_status_file
{
	my ($fname) = @_;
	return unless (-f $fname);
	my $fh = gzopen($fname);

	my %count = (); # To fixup current feature counts for snapshot records.

	while (<$fh>) {
		chomp;
		my @fields = split(/\|/);
		my %f = ();
		foreach (@fields) {
			my ($name, $val) = split(chr(166));
			$f{$name} = $val;
		}
		#event_type¦Login|feature¦RESOLVE|feature_version¦6.0|host¦ENWS08503896|license_start¦3666996224000|product¦LOUCLIENT-PC;nethasp|
		# server¦LOUCLIENT-PC|time¦3690061873000|used_licenses¦1|user¦en34420
		# Login	02/12/15	16:01:55	GAP-v9.0	co24672	NoAccount	ENVS0Z0C	04721		1
		my @f = ($f{event_type});
		
		my $iso = epoch2iso($f{license_start});
		push(@f, iso2dmy(substr($iso,0,10)), substr($iso,11));
		push(@f, $f{feature}.'-'.$f{feature_version});
		$count{$f{feature}} += $f{used_licenses};
		push(@f, $f{user}, 'NoAccount', $f{host}, '04721', '', $count{$f{feature}});
		my $line = join("\t", @f);
		push(@{$lines{$iso}}, $line);
		printf(STDERR "%s\n", $line) if(defined($opt_verbose));
	}
	close($fh);
	#warn("Done parsing $fname");
}
